/***************************************************************************
                          app.c  -  description
                             -------------------
    begin                : Wed Jul  7 07:29:49 IST 2004
    copyright            : (C) 2004 by Hans Mathew
    email                : hansprog@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*** gviewzip-app */

#include <config.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <sys/wait.h>

#define NDEBUG
#include <assert.h>

#include "app.h"
#include "menus.h"
#include "main.h"

#ifdef DEBUG
static gchar unzipcommand[PATHSIZE];
#endif

static gchar abspath[PATHSIZE];			/* Destination path and filename of file to be extracted */
static gchar filepath[PATHSIZE];		/* Path and filename of file in zipped file */	
static gchar *unzipflags1 = "-o";
static gchar *unzipflags2 = "-qq";
static gchar *exdirflag = "-d";

/* To change the order of columns, change its order in the indices 	*/
/* To remove columns, set indices to -1, and adjust CLIST_NUM_COLS 	*/
static gint indices[CLIST_MAX_COLS] = {1,
															2,
															3,
															4,
															0,
															5,
															-1,
															-1,
															-1,
															-1,
															-1,
															-1};
/* Original order is modified date, crc32, compressedsize, uncompressedsize, filename, path	*/
/* If COLUMNNONE do not create column.																						    			*/
static enum enumcolumn columns[CLIST_MAX_COLS] = {MODDATE,
															CRC32,
															SIZEZIP,
															SIZEUNZIP,
															FILENAME,
															PATH,
															COLUMNNONE,
															COLUMNNONE,
															COLUMNNONE,
															COLUMNNONE,
															COLUMNNONE,
															COLUMNNONE};
/* Title of columns */
static gchar *titles[CLIST_MAX_COLS] = {"Mod. Date",
															"Crc 32",
															"Size(Zip)",
															"Size(Unzip)",
															"File name",
															"Path",
															"",
															"",
															"",
															"",
															"",
															""};
/* Width of each column */
static gint widths[CLIST_MAX_COLS] = {110,
															60,
															60,
															80,
															100,
															150,
															0,
															0,
															0,
															0,
															0,
															0};
/* Value type of each column */
static enum enumvaluetype valuetypes[CLIST_MAX_COLS] = {
															FILEDATE,
															NUMBER,
															NUMBER,
															NUMBER,
															STRING,
															STRING,
															VALUENONE,
															VALUENONE,
															VALUENONE,
															VALUENONE,
															VALUENONE,
															VALUENONE};
/* Ascending or descending */
static GtkSortType sorttype[CLIST_MAX_COLS] = {GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING,
															GTK_SORT_ASCENDING};
static enum enumdateformat dateformat = GENERIC;
/* Is the column left justified? */
static enum enumflag leftjustify[CLIST_MAX_COLS] = {YES,
															NO,
															NO,
															NO,
															YES,
															YES,
															NO,
															NO,
															NO,
															NO,
															NO,
															NO};

static gint sep_relpath_filename(gchar *relpath, gchar **path, gchar **filename);
static gint find_file_date_time(gchar *extra, gint extrafieldlength, time_t *filetime);
static gint get_date_string(gchar *datestring, gint buflen, gint year, gint month, gint day, gint hour, gint min, gint sec, enum enumdateformat format);
static gint get_abs_filepath(gchar *abspath, GtkWidget *window, gint row);
static gint get_filepath(GtkWidget *window, gint row, gchar *path);
static gint extract_view(gint *flag, gint view);
static gint remove_abs_filepath(gchar *absfilename);
static gint fill_clist(GtkWidget* clist, const gchar *sortfile, enum enumcolumn sortcolumn);

static gint compare_func(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2);

static gint delete_event_cb(GtkWidget* w, GdkEventAny* e, gpointer data);
static void button_pressed_cb(GtkWidget* window, GdkEventButton* e, gpointer data);
static void click_column_cb(GtkWidget* window, gint column, gpointer data);
static void destroy_notify(gpointer data);
static void file_exists_reply_cb(gint reply, gpointer data);

/* Find the column at the index */
enum enumcolumn findcolumn(gint findindex)
{
	gint i;

	for(i = 0; i < CLIST_MAX_COLS; i++)
	{
		if(indices[i] == findindex)
		{
			return i;
		}
	}
	return COLUMNNONE;
}

/* Used to sort numbers and dates */
static gint
compare_func(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
	enum enumcolumn tempcolumn;
	gint result = 0;
	GtkCListRow *row1 = (GtkCListRow *)ptr1;
	GtkCListRow *row2 = (GtkCListRow *)ptr2;
	struct strrowdata *data1 = (struct strrowdata *)row1->data;
	struct strrowdata *data2 = (struct strrowdata *)row2->data;
	const gchar *text1 = GTK_CELL_TEXT(row1->cell[clist->sort_column])->text;
	const gchar *text2 = GTK_CELL_TEXT(row2->cell[clist->sort_column])->text;

	tempcolumn = columns[findcolumn(clist->sort_column)];
	switch(valuetypes[tempcolumn])
	{
		case FILEDATE:
			result = strcmp(data1->datestring, data2->datestring);
			break;
		case STRING:
			/* Handled automatically */
			assert(0);
			break;
		case NUMBER:
			result = atoi(text2) - atoi(text1);
			break;
		case VALUENONE:
			g_warning(_("Invalid value type"));
			assert(0);
	}
	return result;
}

gint
openzipfile(const gchar *filename, GtkWidget *app, GtkWidget *clist)
{
		enum enumcolumn tempcolumn;
		int ret;
		GtkWidget *widget;

		ret = clear(clist);
		assert(ret != 0);

		if(strlen(filename) < sizeof(zipfilepath))
		{
			strncpy(zipfilepath, filename, sizeof(zipfilepath));

			/* Find the name of the zip file */			
			zipfilename = strrchr(zipfilepath, '/');
			if(zipfilename == NULL)
			{
				zipfilename = zipfilepath;
			}
			else
			{
				zipfilename++;
			}

			gtk_window_set_title(GTK_WINDOW(app), filename);
		}
		else
		{
    	g_warning(_("File path too long"));
			gtk_window_set_title(GTK_WINDOW(app), "gviewzip");
			zipfilepath[0] = '\0';
			return 0;
		}

		/* Initially sort by file name */
		tempcolumn = findcolumn(0);
		if(fill_clist(clist, filename, tempcolumn) == 0)
		{
    	widget = gnome_ok_dialog_parented(_("Could not open the zip file"), GTK_WINDOW(app));
			gtk_window_set_title(GTK_WINDOW(app), "gviewzip");
			zipfilepath[0] = '\0';
			return -1;		
		}
		else
		{
			gtk_window_set_title(GTK_WINDOW(app), filename);
		}

		return 1;
}

GtkWidget* 
gviewzip_app_new(const gchar *zipfile,
              const gchar* geometry)
{
  GtkWidget* app;
  GtkWidget* status;
  GtkWidget* frame;
  GtkWidget* clist;
  GtkWidget* scrollwnd;
	gchar *sortedtitles[CLIST_MAX_COLS];
	enum enumcolumn tempcolumn;
	gint i, openret;
	gint x, y, w, h;

	x = y = w = h = 0;

  /*** gviewzip-widgets */
  app = gnome_app_new(PACKAGE, _("Gviewzip"));
  assert(app != NULL);

  frame = gtk_frame_new(NULL);

	for(i = 0; i < CLIST_NUM_COLS; i++)
	{
		tempcolumn = columns[findcolumn(i)];
		assert(tempcolumn != COLUMNNONE);

		sortedtitles[i] = titles[tempcolumn];
	}

	clist = gtk_clist_new_with_titles(CLIST_NUM_COLS, sortedtitles);
	assert(clist != NULL);
	for(i = 0; i < CLIST_NUM_COLS; i++)
	{
		tempcolumn = columns[findcolumn(i)];
		assert(tempcolumn != COLUMNNONE);

		if(leftjustify[tempcolumn] == NO)
		{
	  	gtk_clist_set_column_justification(GTK_CLIST(clist), i, GTK_JUSTIFY_RIGHT);
		}
  	gtk_clist_set_column_width(GTK_CLIST(clist), i, widths[tempcolumn]);
	}

  scrollwnd = gtk_scrolled_window_new(NULL, NULL);
	/* Show scroll bars only when required */
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  gtk_window_set_policy(GTK_WINDOW(app), FALSE, TRUE, FALSE);
  gtk_window_set_default_size(GTK_WINDOW(app), 350, 350);
  gtk_window_set_wmclass(GTK_WINDOW(app), "zip", "Gviewzip");

  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

  gtk_container_add(GTK_CONTAINER(scrollwnd), clist);
  gtk_container_add(GTK_CONTAINER(frame), scrollwnd);

  gnome_app_set_contents(GNOME_APP(app), frame);

  status = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER);

  gnome_app_set_statusbar(GNOME_APP(app), status);

	/* Assume gviewzip_app_new is called only once per instance */
	mtdata.app = app;
	mtdata.clist = clist;
  gviewzip_install_menus_and_toolbar(app, &mtdata);

  /* gviewzip-widgets ***/
  
  /*** gviewzip-signals */
  gtk_signal_connect(GTK_OBJECT(app),
                     "delete_event",
                     GTK_SIGNAL_FUNC(delete_event_cb),
                     clist);

  gtk_signal_connect(GTK_OBJECT(clist),
                     "button_press_event",
                     GTK_SIGNAL_FUNC(button_pressed_cb),
                     app);

  gtk_signal_connect(GTK_OBJECT(clist),
                     "click_column",
                     GTK_SIGNAL_FUNC(click_column_cb),
                     app);

  /* gviewzip-signals ***/

  /*** gviewzip-geometry */
  if (geometry != NULL)
    {
      if ( gnome_parse_geometry( geometry, 
                                 &x, &y, &w, &h ) ) 
        {
          if (x != -1)
            {
              gtk_widget_set_uposition(app, x, y);
            }

          if (w != -1) 
            {
              gtk_window_set_default_size(GTK_WINDOW(app), w, h);
            }
        }
      else 
        {
          g_error(_("Could not parse geometry string `%s'"), geometry);
        }
    }

  /* gviewzip-geometry ***/

  app_list = g_slist_prepend(app_list, app);

	assert(indices[(int)FILENAME] != -1);
	gtk_clist_set_sort_column(GTK_CLIST(clist), indices[(int)FILENAME]);
	gtk_clist_set_auto_sort(GTK_CLIST(clist), 1);

	if(zipfile != NULL && zipfile[0] != '\0')
	{
		openret = openzipfile(zipfile, app, clist);
		assert(openret!= 0);
	}

  return app;
}

void
gviewzip_app_close(GtkWidget* app)
{
  g_return_if_fail(GNOME_IS_APP(app));

	if(memcmp(tempdir, tempunzipbasedir, strlen(tempdir)) == 0)
	{
		rmdir(tempunzipbasedir);
	}

  app_list = g_slist_remove(app_list, app);

  gtk_widget_destroy(app);

  if (app_list == NULL)
    {
      /* No windows remaining */
      gtk_main_quit();
    }
}

/*** gviewzip-quit */
gint
delete_event_cb(GtkWidget* window, GdkEventAny* e, gpointer data)
{
	gint ret;

  ret = clear(data);
  assert(ret != 0);

	gviewzip_app_close(window);

  /* Prevent the window's destruction, since we destroyed it
   * ourselves with gviewzip_app_close()
   */
  return 1;
}
/* gviewzip-quit ***/

static gint
extract_view(gint *flag, gint view)
{
	pid_t procunzip = -1, proceditor, waitret;
	gint status;

	/* Check if filepath is valid */
	if((gchar *)filepath == NULL)
	{
		g_warning(_("File path is not valid"));
		assert(0);
		return 0;
	}
	if(*(gchar *)filepath == '\0')
	{
		g_warning(_("File path is not valid"));
		assert(0);
		return 0;
	}

	/* Check if temp and unzip directories are valid */
	if(memcmp(tempdir, tempunzipbasedir, strlen(tempdir)) != 0)
	{
		g_warning(_("Temporary unzip base path is not valid"));
		assert(0);
		return 0;
	}

	/* Check if absolute path is valid */
	if(memcmp(abspath, tempunzipbasedir, strlen(tempunzipbasedir)) != 0)
	{
		g_warning(_("Absolute path is not valid"));
		assert(0);
		return 0;
	}

	if(memcmp(abspath + strlen(tempunzipbasedir) + 1, zipfilename, strlen(zipfilename)) != 0)
	{
		g_warning(_("Absolute path is not valid"));
		assert(0);
		return 0;
	}

	if(memcmp(abspath + strlen(tempunzipbasedir) + 1 + strlen(zipfilename) + 1, filepath, strlen(filepath)) != 0)
	{
		g_warning(_("Absolute path is not valid"));
		assert(0);
		return 0;
	}

	/* No editor found */
	if(editorexec[0] == '\0')
	{
		g_warning(_("No editors found. Use Settings->Preferences"));
		return -1;
	}

#ifdef DEBUG
	/* Show the unzip command */
	snprintf(unzipcommand, PATHSIZE, "%s %s %s %s %s %s %s", unzipexec, unzipflags1, unzipflags2, zipfilepath, filepath, exdirflag, tempunzipdir);
	g_print("%s\n",unzipcommand);
	g_print("Viewed = %d\n", *flag & VIEWEDFILE);
#endif

	/* If not previously extracted, extract file */
	if((*flag & VIEWEDFILE) == 0)
	{
		procunzip = fork();

		if(procunzip == -1)
		{	
			g_warning(_("Error during unzip fork"));
			assert(0);
			return 0;
		}
		else if(procunzip == 0)
		{
			execl(unzipexec, unzipexec, unzipflags1, unzipflags2, zipfilepath, filepath, exdirflag, tempunzipdir, NULL);
		}
  }

	/* No need to extract this file again for viewing */
	*flag |= VIEWEDFILE;

	/* Extract only. Do not view */
	if(view == 0)
	{
		return 1;
	}

	if(procunzip != -1)
	{
		waitret = waitpid(procunzip, &status, 0);
		assert(waitret != -1);
	}

	assert(memcmp(tempdir, tempunzipbasedir, strlen(tempdir)) == 0);
	assert(memcmp(abspath, tempunzipbasedir, strlen(tempunzipbasedir)) == 0);
	assert(memcmp(abspath + strlen(tempunzipbasedir) + 1, zipfilename, strlen(zipfilename)) == 0);
	assert(memcmp(abspath + strlen(tempunzipbasedir) + strlen(zipfilename) + 1 + 1, filepath, strlen(filepath)) == 0);

	proceditor = fork();

	if(proceditor == -1)
	{
		g_warning(_("Error during editor fork"));
		assert(0);
		return 0;
	}
	else if(proceditor == 0)
	{
		execl((gchar *)(editorexec), (gchar *)(editorexec), abspath, NULL);
	}

	return 1;
}

static void
file_exists_reply_cb(gint reply, gpointer data)
{
	gint ret;

	assert((gchar *)data != NULL);

	/* Did the user ask to overwrite the file? */
	if(reply == GNOME_YES)
	{
		ret = extract_view(data, 1);
		assert(ret != 0);
	}
}

/* Given a column, return its position in clist */
static gint
get_column(enum enumcolumn column)
{
	gint i;

	for(i = 0; i < CLIST_NUM_COLS; i++)
	{
		if(columns[i] == column)
		{
			return indices[i];
		}
	}
	return -1;
}

/* Create the absolute path to which zipped file will be extracted 							*/
/* Assumes that globals tempdir, tempunzipbasedir and zipfilename are valid					*/
/* Each file will be extracted to a path relative to tempunzipbasedir + "/" + zipfilename	*/
/* (Contents of gvzsrc.zip for eg.will be extracted to /root/gviewzip_files/gvzsrc.zip/) 	*/
static gint
get_abs_filepath(gchar *abspath, GtkWidget *window, gint row)
{
	gint ret;

	/* Check if unzip temp directroy is valid */
	if(memcmp(tempdir, tempunzipbasedir, strlen(tempdir)) != 0)
	{
		g_warning(_("Temporary unzip base path is not valid"));
		assert(0);
		return 0;
	}

	if((strlen(tempunzipbasedir) + 1 + strlen(zipfilename)) < PATHSIZE)
	{
		strcpy(tempunzipdir, tempunzipbasedir);
		strcat(tempunzipdir, "/");
		strcat(tempunzipdir, zipfilename);		
	}

	if((strlen(tempunzipdir) + 1 + strlen(filepath)) < PATHSIZE)
	{
		strcpy(abspath, tempunzipdir);
  	ret = get_filepath(window, row, filepath);
	assert(ret != 0);

		/* Add a / if required */
		if(*filepath != '/')
		{
			strcat(abspath, "/");
		}

		strcat(abspath, filepath);
	}

	return 1;
}

/* Get path + filename from clist */
static gint
get_filepath(GtkWidget *window, gint row, gchar *path)
{
	gint pathcol, filenamecol;
	gchar *pathfromcol = NULL, *filenamefromcol = NULL;

	/* Add the path of the file to be extracted */
	pathcol = get_column(PATH);
	gtk_clist_get_text(GTK_CLIST(window), row, pathcol, &pathfromcol);

	/* Add the file name of the file to be extracted */
	filenamecol = get_column(FILENAME);
	gtk_clist_get_text(GTK_CLIST(window), row, filenamecol, &filenamefromcol);

	snprintf(path, PATHSIZE, "%s%s%s", pathfromcol, strlen(pathfromcol) > 0 ? "/" : "", filenamefromcol);

	return 1;
}

static void
button_pressed_cb(GtkWidget* window, GdkEventButton* e, gpointer data)
{
	FILE *fp;
	gint selectedrow = 0, *flag;
	struct strrowdata *rowdata = NULL;
	gint ret;

  /* Detect double click */
  if(e->type == GDK_2BUTTON_PRESS)
	{
		if(gtk_clist_get_selection_info(GTK_CLIST(window), (gint)ceil(e->x), (gint)ceil(e->y), &selectedrow, NULL) == 0)
		{
			g_warning(_("Could not find double clicked row"));
			assert(0);
			return;
		}

		if(memcmp(tempdir, tempunzipbasedir, strlen(tempdir)) != 0)
		{
			g_warning(_("Temporary unzip base path is not valid"));
			assert(0);
			return;
		}

		/* Get absolute path of file to be extracted */	
		ret = get_abs_filepath(abspath, window, selectedrow);
		assert(ret != 0);

		rowdata = gtk_clist_get_row_data(GTK_CLIST(window), selectedrow);
		flag = &rowdata->flag;

		if((*flag & ENCRYPTEDFILE) == ENCRYPTEDFILE)
		{
			gnome_ok_dialog_parented(_("Cannot view encrypted files"), GTK_WINDOW(data));
			return;
		}

		if((*flag & VIEWEDFILE) == 0 && (fp = fopen(abspath, "r")) != NULL)
		{
			ret = fclose(fp);
			assert(ret == 0);
	
			gnome_question_dialog_modal_parented(
          _("Temporary file already exists. Overwrite?"),
					file_exists_reply_cb,
					flag, GTK_WINDOW(data));

			return;
		}

		ret = extract_view(flag, 1);
		assert(ret != 0);
	}
}

/* Sort columns when column titles are clicked */
static void
click_column_cb(GtkWidget* window, gint column, gpointer data)
{
	enum enumcolumn tempcolumn, currcol;
	gint i;

	tempcolumn = columns[findcolumn(column)];
	assert(tempcolumn != COLUMNNONE);

	for(i = 0; i < CLIST_NUM_COLS; i++)
	{
		currcol = findcolumn(i);
		if(tempcolumn == currcol)
		{
			sorttype[currcol] = (sorttype[currcol] == GTK_SORT_ASCENDING) ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING;
		}
		else
		{
			sorttype[currcol] = GTK_SORT_ASCENDING;
		}
	}

	gtk_clist_set_sort_column(GTK_CLIST(window), column);
	gtk_clist_set_sort_type(GTK_CLIST(window), sorttype[tempcolumn]);

  /* If value type is string use default compare function */
	if(valuetypes[tempcolumn] != STRING)
	{
		gtk_clist_set_compare_func(GTK_CLIST(window), compare_func);
	}

	gtk_clist_sort(GTK_CLIST(window));

	if(valuetypes[tempcolumn] != STRING)
	{
		gtk_clist_set_compare_func(GTK_CLIST(window), NULL);
	}
}

/* Remove the directory along with its path */
static gint
remove_abs_filepath(gchar *absfilepath)
{
	gchar *slash;

	/* Make sure that the path is absolute */
	if(*absfilepath != '/')
	{
		g_warning("Path to be removed is not abolute");
		assert(0);
		return 0;
	}

	if(memcmp(tempdir, tempunzipdir, strlen(tempdir)) != 0)
	{
		g_warning("Temporary unzip base path is invalid");
		assert(0);
		return 0;
	}

	/* Make sure that we are deleting files from temp dir */
	if(memcmp(absfilepath, tempunzipdir, strlen(tempunzipdir)) != 0)
	{
		g_warning("Absolute path is not valid");
		assert(0);
		return 0;
	}

	/* Error deleting file */
	if(unlink(absfilepath) == -1)
	{
		g_warning("Could not remove file: %s", absfilepath);
		assert(0);
		return 0;
	}

	while(strcmp(absfilepath, tempunzipdir))
	{
		slash = strrchr(absfilepath, '/');
		if(slash != NULL)
		{
			*slash = '\0';
		}

		/* Try deleting the directories in the path */
		if(rmdir(absfilepath) == -1)
		{
			/* May be the directory is not empty */
			break;
		}
	}

	return 1;
}

/* Remove all rows from the clist */
gint
clear(GtkWidget* clist)
{
	struct strrowdata *rowdata;
	gint ret;

	/* Avoid flicker */
	gtk_clist_freeze(GTK_CLIST(clist));

	/* Process each row */
	while(g_list_first(GTK_CLIST(clist)->row_list) != NULL)
	{
		ret = get_abs_filepath(abspath, clist, 0);
		assert(ret != 0);

		/* Delete viewed files */
		rowdata = (struct strrowdata *)gtk_clist_get_row_data(GTK_CLIST(clist), 0);

		if((rowdata->flag & VIEWEDFILE) == VIEWEDFILE)
		{
			ret = remove_abs_filepath(abspath);
			assert(ret != 0);
		}
		
		/* Remove file from clist */
		gtk_clist_remove(GTK_CLIST(clist), 0);
	}

	gtk_clist_thaw(GTK_CLIST(clist));

  return 1;
}

/* Get date in specified format */
static gint
get_date_string(gchar *datestring, gint buflen, gint year, gint month, gint day, gint hour, gint min, gint sec, enum enumdateformat format)
{
	char *months[12]= {	"Jan",
											"Feb",
											"Mar",
											"Apr",
											"May",
											"Jun",
											"Jul",
											"Aug",
											"Sep",
											"Oct",
											"Nov",
											"Dec",
									};

	switch(format)
	{
		case GENERIC:
			snprintf(datestring, buflen, "%02d %s %04d %02d:%02d", day, months[month - 1], year, hour, min);
			dateformat = GENERIC;
			break;
		case AMERICAN:
			snprintf(datestring, buflen, "%02d/%02d/%04d %02d:%02d", month, day, year, hour, min);
			dateformat = AMERICAN;
			break;
		case BRITISH:
			snprintf(datestring, buflen, "%02d/%02d/%04d %02d:%02d", day, month, year, hour, min);
			dateformat = BRITISH;
			break;
		case COMPARE:
			/* This would return the date string so that dates can be soted by strcmp */
			snprintf(datestring, buflen, "%04d%02d%02d%02d%02d", year, month, day, hour, min);
			dateformat = COMPARE;
			break;
		default:
			g_warning("Unsupported date format");
			assert(0);
	}

	return 1;
}

/* Extra field is in the format [HeaderID1][DataSize1][Data1]...[HeaderIDn][DataSizen][Datan]	*/
static gint
find_file_date_time(gchar *extra, gint extrafieldlength, time_t *filetime)
{
	gchar *tempextra;
	int found = -1;
	gushort extrablocklen = 0;

	/* Find the file time from the extra field */
	for(tempextra = extra; tempextra - extra < extrafieldlength; tempextra += extrablocklen + EXTRAHEADERSIZE)
	{
		extrablocklen = *((gushort *)(tempextra + EXTRADATASIZEOFF));
		if(*(gushort *)tempextra == 0x5455)
		{
			assert((tempextra - extra + 9) < extrafieldlength);
			assert(extrablocklen > 4);

			*filetime = *(time_t *)(tempextra + 5);
			found = 1;
		}
	}

	return found;	
}

/* Seperate relative path in zip file to path and filename */
static gint
sep_relpath_filename(gchar *fullpath, gchar **path, gchar **filename)
{
		char *tempfilename;

		/* Seperate fullpath into path and filename */
		tempfilename = strrchr(fullpath, '/');
		if(tempfilename == NULL)
		{
			*filename = fullpath;
			*path = "";
		}
		else
		{
			*tempfilename++ = '\0';
			*filename = tempfilename;
			*path = fullpath;
		}

		return 1;
}

/* This would be called for each row. Free row data */
static void
destroy_notify(gpointer data)
{
	g_free(data);
}

/* Adds each compressed file to clist */
static gint
fill_clist(GtkWidget* clist, const gchar *sortfile, enum enumcolumn sortcolumn)
{
  struct strlocalfileheader localfileheader;
	struct strrowdata *rowdata;
  FILE *zipfile;					/* Zip file */
	enum enumcolumn tempcolumn;
	struct tm *tmfiletime = NULL;
	time_t timetfiletime;
	gchar *text[CLIST_NUM_COLS];	/* To be appended to clist for each row */
  gchar *fullpath; 				/* Buffer for filename */
	gchar *extra = NULL;
	gchar *filename = NULL;			
	gchar *path = NULL;
	gchar datestring[DATESTRSIZE];
  gchar crc32[10];
  gchar compressedsize[10];
  gchar uncompressedsize[10];
  gint nret = 1;
	gint i, ret, found, index;
	gint year, month, day, hour, minute, second;
	size_t count;
	
	totalsize = 0;
	zippedfiles = 0;

 	/* Open zip file */
	zipfile = fopen(sortfile, "r");
	if(zipfile == 0)
	{
		return 0;
	}

	gtk_clist_freeze(GTK_CLIST(clist));
  while(TRUE)
	{
   	memset(&localfileheader, 0, sizeof(localfileheader));

   	/* Read local file header */
   	READBYTES(&localfileheader.fileheadersignature, sizeof(localfileheader.fileheadersignature))
   	if(localfileheader.fileheadersignature != 0x04034b50)
   	{
			/* Reached end? */
   		break;
   	}

	  READBYTES(&localfileheader.versionneededtoextract, sizeof(localfileheader.versionneededtoextract))
   	READBYTES(&localfileheader.generalpurposebitflag, sizeof(localfileheader.generalpurposebitflag))
   	READBYTES(&localfileheader.compressionmethod, sizeof(localfileheader.compressionmethod))
   	READBYTES(&localfileheader.lastmodfiletime, sizeof(localfileheader.lastmodfiletime))
   	READBYTES(&localfileheader.lastmodfiledate, sizeof(localfileheader.lastmodfiledate))
   	READBYTES(&localfileheader.crc32, sizeof(localfileheader.crc32))
   	READBYTES(&localfileheader.compressedsize, sizeof(localfileheader.compressedsize))
   	READBYTES(&localfileheader.uncompressedsize, sizeof(localfileheader.uncompressedsize))
   	READBYTES(&localfileheader.filenamelength, sizeof(localfileheader.filenamelength))
   	READBYTES(&localfileheader.extrafieldlength, sizeof(localfileheader.extrafieldlength))

  	fullpath = g_malloc0(localfileheader.filenamelength + 1);
	  if(fullpath == NULL)
   	{
			g_error(_("Not enough memory\n"));
			nret = 0;
   		break;
   	}

   	/* Read filename */
		count = fread(fullpath, 1, (size_t)localfileheader.filenamelength, zipfile);
		if(count != localfileheader.filenamelength)
		{
			g_free(fullpath);
			nret = 0;
			break;
		}

		if(localfileheader.extrafieldlength > 0)
		{
   		extra = g_malloc0((gulong)localfileheader.extrafieldlength);
	  	if(extra == NULL)
   		{
				g_error(_("Not enough memory\n"));
				g_free(fullpath);
				nret = 0;
  	 		break;
   		}

			/* Read extra bytes */
			count = fread(extra, 1, (size_t)localfileheader.extrafieldlength, zipfile);
 			if(count != (size_t)localfileheader.extrafieldlength)
 			{
				g_free(fullpath);
				g_free(extra);
				nret = 0;
				break;
 			}
		}
		
		ret = sep_relpath_filename(fullpath, &path, &filename);
		assert(ret != 0);

		/* Should be a directory */
		if(strlen(filename) < 1)
		{
			g_free(fullpath);
			g_free(extra);
			continue;
		}

	  	memset(&timetfiletime, 0, sizeof(timetfiletime));
		found = find_file_date_time(extra, (gint)localfileheader.extrafieldlength, &timetfiletime);

		if(found == 1 && localfileheader.extrafieldlength > 0)
		{
			tmfiletime = localtime(&timetfiletime);
			assert(tmfiletime != NULL);
	
			year = tmfiletime->tm_year + 1900;
			month = tmfiletime->tm_mon + 1;
			day = tmfiletime->tm_mday;
			hour = tmfiletime->tm_hour;
			minute = tmfiletime->tm_min;
			second = tmfiletime->tm_sec;
		}
		else
		{
			year = GETYEARFROMDOSDATE(localfileheader.lastmodfiledate)
			month = GETMONTHFROMDOSDATE(localfileheader.lastmodfiledate)
			day = GETDAYFROMDOSDATE(localfileheader.lastmodfiledate)
			hour = GETHOURFROMDOSTIME(localfileheader.lastmodfiletime)
			minute = GETMINUTEFROMDOSTIME(localfileheader.lastmodfiletime)
			second = GETMINUTEFROMDOSTIME(localfileheader.lastmodfiletime)
		}

	  snprintf(crc32, sizeof(crc32), "%0x", localfileheader.crc32);
	  snprintf(compressedsize, sizeof(compressedsize), "%u", localfileheader.compressedsize);
	  snprintf(uncompressedsize, sizeof(uncompressedsize), "%u", localfileheader.uncompressedsize);

		ret = get_date_string(datestring, DATESTRSIZE, year, month, day, hour, minute, second, GENERIC);
		assert(ret != 0);

		/* Add files to clist */
		for(i = 0; i < CLIST_NUM_COLS; i++)
		{
			tempcolumn = columns[findcolumn(i)];
			assert(tempcolumn != COLUMNNONE);

			switch(tempcolumn)
			{
			case MODDATE:
				text[i] = datestring;
			  break;
			case CRC32:
				text[i] = crc32;
			  break;
			case SIZEZIP:
				text[i] = compressedsize;
			  break;
			case SIZEUNZIP:
				text[i] = uncompressedsize;
			  break;
			case FILENAME:
				text[i] = filename;
			  break;
			case PATH:
				text[i] = path;
			  break;
			default:
				assert(0);
			}
		}

		index = gtk_clist_append(GTK_CLIST(clist), text);

		/* Set data for each column */
		rowdata = g_malloc0((guint)sizeof(struct strrowdata));
		ret = get_date_string(rowdata->datestring, DATESTRSIZE, year, month, day, hour, minute, second, COMPARE);

		/* LSB is encrypted flag */
		if((localfileheader.generalpurposebitflag & 1) == 1)
		{
			rowdata->flag = ENCRYPTEDFILE;
		}
		else
		{
			rowdata->flag = 0;
		}
		gtk_clist_set_row_data_full(GTK_CLIST(clist), index, rowdata, destroy_notify);

		g_free(fullpath);
		g_free(extra);

		totalsize += localfileheader.uncompressedsize;
		zippedfiles++;

		if((localfileheader.generalpurposebitflag & 8) == 8)
		{
			g_warning(_("Unsupported zip format\n"));
			clear(clist);
		  nret = 0;
		  break;
	 	}

  	if(fseek(zipfile, (gint)localfileheader.compressedsize, SEEK_CUR) != 0)
		{
			g_warning(_("Error skipping compressed data\n"));
			clear(clist);
			nret = 0;
			break;
		}
	}
	
	gtk_clist_thaw(GTK_CLIST(clist));

	/* Close zip file */
	fclose(zipfile);

  return nret;
}

/* gviewzip-app ***/
