/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Wed Jul  7 07:29:49 IST 2004
    copyright            : (C) 2004 by Hans Mathew
    email                : hansprog@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*** gviewzip */

#include <config.h>
#include <gnome.h>
#include <sys/stat.h>

#include "app.h"
#include "menus.h"

#define NDEFEXTERN
#include "main.h"

static gint session_die(GnomeClient* client, gpointer client_data);

static gint save_session(GnomeClient *client, gint phase,
                         GnomeSaveStyle save_style,
                         gint is_shutdown, GnomeInteractStyle interact_style,
                         gint is_fast, gpointer client_data);

static gint checkdir(gchar *dirpath);

/*** gviewzip-popttable */
static gchar* geometry = NULL;

struct poptOption options[] = {
  {
    "geometry",
    '\0',
    POPT_ARG_STRING,
    &geometry,
    0,
    N_("Specify the geometry of the main window"),
    N_("GEOMETRY")
  },
  {
    NULL,
    '\0',
    0,
    NULL,
    0,
    NULL,
    NULL
  }
};
/* gviewzip-popttable ***/

gint close_cb(GtkWidget *widget, gpointer data)
{
	gviewzip_app_close(data);	
	
	return TRUE;			
}

gint show_error_dialog(GtkWidget *app, gchar *message)
{
	GtkWidget *errordialog;

 	errordialog = gnome_error_dialog_parented(buff, GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(errordialog), TRUE);
  gtk_signal_connect (GTK_OBJECT(errordialog), "close",
                      GTK_SIGNAL_FUNC (close_cb), app);

	return TRUE;
}

gint
main(gint argc, gchar* argv[])
{
  /*** gviewzip-parsing */
  GtkWidget* app;
  poptContext pctx;
  const char** args;
  GnomeClient* client;
	gint error = 0;

	/* Initialize globals */
	unzipexec = "/usr/bin/unzip";
	tempdir = "/tmp/";
	tempunzipbasedir = "/tmp/gviewzip_files";
	app_list = NULL;
	totalsize = zippedfiles = 0;
	editorexec[0] = '\0';
	zipfilepath[0] = '\0';

	memset(&pctx, 0, sizeof(pctx));

  bindtextdomain(PACKAGE, GNOMELOCALEDIR);
  textdomain(PACKAGE);

  gnome_init_with_popt_table(PACKAGE, VERSION, argc, argv,
                             options, 0, &pctx);

  /* Argument parsing */
  args = poptGetArgs(pctx);

 	/* Get the name of zip file */
	if(args != NULL && argv[0] != NULL)
	{
		if(strlen(args[0]) < sizeof(zipfilepath))
		{
				strcpy(zipfilepath, args[0]);
		}
	}

  poptFreeContext(pctx);
  /* gviewzip-parsing ***/

  /* Session Management */
  
  /*** gviewzip-client */
  client = gnome_master_client ();
  gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
                      GTK_SIGNAL_FUNC (save_session), argv[0]);
  gtk_signal_connect (GTK_OBJECT (client), "die",
                      GTK_SIGNAL_FUNC (session_die), NULL);
  /* gviewzip-client ***/

  
  /* Main app */
  app = gviewzip_app_new(zipfilepath, geometry);

	/* See if unzip is executable */
	if(is_executable(unzipexec) != 1)
	{
		snprintf(buff, BUFSIZE, _("You cannot execute %s"), unzipexec);
		show_error_dialog(app, buff);
		error = 1;
	}
	/* Make sure that the temp diectory exists */
	else if(checkdir(tempdir) != 1)
	{
		snprintf(buff, BUFSIZE, _("%s directory not accessible"), tempdir);
		show_error_dialog(app, buff);
		error = 1;
	}

	if(error == 0)
	{
		preferences_func(NULL, &mtdata);
	}

  /*** gviewzip-main */
  gtk_widget_show_all(app);

	if(memcmp(tempdir, tempunzipbasedir, strlen(tempdir)) == 0)
	{
		mkdir(tempunzipbasedir, 0);
	}

  gtk_main();

  return 0;
  /* gviewzip-main ***/
}

/*** gviewzip-save-session */
static gint
save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
              gint is_shutdown, GnomeInteractStyle interact_style,
              gint is_fast, gpointer client_data)
{
  gchar** argv;
  guint argc;

  argv = g_malloc0(sizeof(gchar*)*3);
  argc = 1;

  argv[0] = client_data;

  if(zipfilepath[0] != '\0')
  {
  	argv[1] = zipfilepath;
    argc = 2;
  }
  
  gnome_client_set_clone_command (client, argc, argv);
  gnome_client_set_restart_command (client, argc, argv);

  return 1;
}
/* gviewzip-save-session ***/

/*** gviewzip-session-die */
static gint 
session_die(GnomeClient* client, gpointer client_data)
{
  gtk_main_quit ();
  return TRUE;
}
/* gviewzip-session-die ***/

/* Checks if a directory exists or not */
gint
checkdir(gchar *dirpath)
{
	struct stat buf;
	int ret;

	ret = stat(dirpath, &buf);
	if(ret == -1)
	{
		return -1;
	}

	if((buf.st_mode & S_IWUSR) != S_IWUSR)
	{
		return 0;
	}
	
	return 1;
}

/* gviewzip ***/
