/***************************************************************************
                          menus.c  -  description
                             -------------------
    begin                : Wed Jul  7 07:29:49 IST 2004
    copyright            : (C) 2004 by Hans Mathew
    email                : hansprog@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*** gviewzip-menus */

#include <config.h>
#include <sys/stat.h>

#define NDEBUG
#include <assert.h>
#include "app.h"
#include "menus.h"
#include "main.h"

static void open_cb(GtkWidget* widget, gpointer data);
static void properties_cb(GtkWidget* widget, gpointer data);
static void preferences_cb(GtkWidget* widget, gpointer data);
static void properties_cb(GtkWidget* widget, gpointer data);
static void close_cb  (GtkWidget* widget, gpointer data);
static void exit_cb   (GtkWidget* widget, gpointer data);
static void about_cb  (GtkWidget* widget, gpointer data);
static void editor_dialog_clicked_cb(GnomeDialog *dialog, gint button_number, gpointer data);
static void zipfile_dialog_clicked_cb(GnomeDialog *dialog, gint button_number, gpointer data);
static GtkWidget* create_file_selection_window(GtkWidget *window, gchar *title, gchar *historyid, gchar **textinentry, GtkSignalFunc dialog_clicked_func_cb, gpointer buffer, gpointer data);
static void entry_focus_out_cb(GtkWidget* widget, GdkEventFocus *event, gpointer data);
static void fileentry_focus_in_cb(GtkWidget* widget, GdkEventFocus *event, gpointer data);

static GnomeUIInfo file_menu [] = {
  GNOMEUIINFO_MENU_OPEN_ITEM(open_cb, NULL),

  GNOMEUIINFO_SEPARATOR,

  GNOMEUIINFO_MENU_CLOSE_ITEM(close_cb, NULL),
  GNOMEUIINFO_MENU_EXIT_ITEM(exit_cb, NULL),

  GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu [] = {
  GNOMEUIINFO_MENU_PROPERTIES_ITEM(properties_cb, NULL),

  GNOMEUIINFO_END
};

static GnomeUIInfo settings_menu [] = {
  GNOMEUIINFO_MENU_PREFERENCES_ITEM(preferences_cb, NULL),

  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu [] = {
  GNOMEUIINFO_HELP ("gviewzip"),
  GNOMEUIINFO_MENU_ABOUT_ITEM(about_cb, NULL),
  
  GNOMEUIINFO_END
};

static GnomeUIInfo menu [] = {
  GNOMEUIINFO_MENU_FILE_TREE(file_menu),
  GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
  GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu),
  GNOMEUIINFO_MENU_HELP_TREE(help_menu),

  GNOMEUIINFO_END
};

static GnomeUIInfo toolbar [] = {
  GNOMEUIINFO_ITEM_STOCK (N_("Open"), N_("Open zip file"), open_cb, GNOME_STOCK_PIXMAP_OPEN),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_STOCK (N_("Properties"), N_("Properties"), properties_cb, GNOME_STOCK_PIXMAP_PROPERTIES),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_STOCK (N_("Preferences"), N_("Preferences"), preferences_cb, GNOME_STOCK_PIXMAP_PREFERENCES),

  GNOMEUIINFO_END
};

void
gviewzip_install_menus_and_toolbar(GtkWidget* app, struct strmtdata *mtdata)
{
  gnome_app_create_toolbar_with_data(GNOME_APP(app), toolbar, mtdata);
  gnome_app_create_menus_with_data(GNOME_APP(app), menu, mtdata);
  gnome_app_install_menu_hints(GNOME_APP(app), menu);
}

static void
zipfile_dialog_clicked_cb(GnomeDialog *dialog, gint button_number, gpointer data)
{
	gint ret;

	/* Exit if cancel button was clicked */
	if(button_number == 1)
	{
		gtk_widget_destroy(GTK_WIDGET(dialog));
		return;
	}

	/* Open the file specified by the user */
	if(strlen(dialogbuf) > 0)
	{
		ret = openzipfile(dialogbuf, GTK_WIDGET(((struct strmtdata *)data)->app), GTK_WIDGET(((struct strmtdata *)data)->clist));
		assert(ret != 0);
	}

	gtk_widget_destroy(GTK_WIDGET(dialog));
}

static void
open_cb(GtkWidget* widget, gpointer data)
{
	GtkWidget *app = ((struct strmtdata *)data)->app;
	GtkWidget *dialog;
	gchar *textinentry = NULL;

	dialog = create_file_selection_window(app, "Select zip file", "gviewzipfiles", &textinentry, zipfile_dialog_clicked_cb, dialogbuf, data);
	gnome_dialog_run(GNOME_DIALOG(dialog));
}

static void
preferences_cb(GtkWidget* widget, gpointer data)
{
	preferences_func(widget, data);
}

/* Used to show the dialog box from main() */
void
preferences_func(GtkWidget* widget, gpointer data)
{
	GtkWidget *app = ((struct strmtdata *)data)->app;
	GtkWidget *dialog;
	gchar *textinentry = NULL;

	dialog = create_file_selection_window(app, "Select editor", "gviewzipeditors", &textinentry, editor_dialog_clicked_cb, editorexec, data);

	/* If an editor is present and called from main(), don't show the dialog */
	if(strlen(textinentry) > 0 && widget == NULL)
	{
		if(strlen(textinentry) < sizeof(editorexec))
		{
			strncpy(editorexec, textinentry, strlen(textinentry));
		}
		gtk_widget_destroy(dialog);

		return;
	}

	gnome_dialog_run(GNOME_DIALOG(dialog));
}

static void
properties_cb(GtkWidget* widget, gpointer data)
{
	GtkWidget *app = ((struct strmtdata *)data)->app;

	/* totalsize and zippedfiles  are calculated in fill_clist */
	snprintf(buff, BUFSIZE, "%ld bytes in %d files", totalsize, zippedfiles);
	gnome_ok_dialog_parented(buff, GTK_WINDOW(app));
}

static void
close_cb(GtkWidget* widget, gpointer data)
{
	gint ret;

	GtkWidget *app = ((struct strmtdata *)data)->app;
	GtkWidget *clist = ((struct strmtdata *)data)->clist;

	/* Just clear the clist */
	ret = clear(clist);
	assert(ret != 0);

	gtk_window_set_title(GTK_WINDOW(app), _("gviewzip"));
	zipfilepath[0] = '\0';
}

static void 
exit_cb(GtkWidget* widget, gpointer data)
{
	clear(((struct strmtdata *)data)->clist);
  gviewzip_app_close(((struct strmtdata *)data)->app);
/*  gtk_main_quit();*/
}

static void 
about_cb(GtkWidget* widget, gpointer data)
{
  static GtkWidget* dialog = NULL;

  if (dialog != NULL) 
  {
    g_assert(GTK_WIDGET_REALIZED(dialog));
    gdk_window_show(dialog->window);
    gdk_window_raise(dialog->window);
  }
  else
  {
    const gchar *authors[] = {
      "Hans Mathew <hansprog@yahoo.com>",
      NULL
			};

	  gchar* logo = gnome_pixmap_file("gviewzip-logo.png");

  	dialog = gnome_about_new (_("Gviewzip"), VERSION,
                                "(C) 2004 Hans Mathew",
                                authors,
                                _("Zip file viewer\nRead COPYING, README and INSTALL for details"),
                                logo);

  	g_free(logo);

  	gtk_signal_connect(GTK_OBJECT(dialog),
                         "destroy",
                         GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                         &dialog);

  	gtk_widget_show(dialog);
	}
}

gint
is_executable(gchar *filename)
{
	struct stat buf;
	gint ret;

	/* Find if the file is executable */
	ret = stat(filename, &buf);
  if((buf.st_mode & S_IFDIR) == S_IFDIR || (buf.st_mode & S_IXUSR) != S_IXUSR)
	{
		return 0;
	}

	return 1;
}

static void editor_dialog_clicked_cb(GnomeDialog *dialog, gint button_number, gpointer data)
{
	/* Exit if cancel button was clicked */
	if(button_number == 1)
	{
		gtk_widget_destroy(GTK_WIDGET(dialog));
		return;
	}

	if(editorexec[0] != '/')
	{
		gnome_ok_dialog_parented(_("Enter path from /"), GTK_WINDOW(dialog));
		return;
	}

	/* Find out if user can execute the file */
	if(is_executable(editorexec) != 1)
	{
		gnome_ok_dialog_parented(_("You cannot execute the file"), GTK_WINDOW(dialog));
		return;
	}

	gtk_widget_destroy(GTK_WIDGET(dialog));
}

/* Set focus to combo when the dialog gets focus */
static void
fileentry_focus_in_cb(GtkWidget* widget, GdkEventFocus *event, gpointer data)
{
	gtk_widget_grab_focus(GTK_WIDGET(data));
}

/* Fill buffer with value when exiting combo 				*/
/* Assume that there are no keyboard accelerators 	*/
static void
entry_focus_out_cb(GtkWidget* widget, GdkEventFocus *event, gpointer data)
{
	gchar *editor;

	editor = gtk_entry_get_text(GTK_ENTRY(widget));
	if(strlen(editor) < PATHSIZE)
	{
		strncpy(data, editor, PATHSIZE);
	}
}

static GtkWidget*
create_file_selection_window(GtkWidget *window, gchar *title, gchar *historyid, gchar **textinentry, GtkSignalFunc dialog_clicked_func_cb, gpointer buffer, gpointer data)
{
  GtkWidget *dialogfileselection;
	GtkWidget *gnomefileentry;
	GtkWidget *gtkentry;
	GtkWidget *gnomeentry;
	GtkWidget *gtkcombo;

	/* The order of the buttons is used in dialog_clicked_cb */
	dialogfileselection = gnome_dialog_new(title, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialogfileselection), GTK_WINDOW(window));

	gnomefileentry = gnome_file_entry_new(historyid, title);

	gtkentry = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(gnomefileentry));

	gnomeentry = GNOME_FILE_ENTRY(gnomefileentry)->gentry;
	gnome_entry_set_max_saved(GNOME_ENTRY(gnomeentry), 5);
	/* Select the first item in the list */
	gtkcombo = GTK_WIDGET(&(GNOME_ENTRY(gnomeentry)->combo));
	gtk_list_select_item(GTK_LIST(GTK_COMBO(gtkcombo)->list), 0);
	gtk_combo_set_use_arrows(GTK_COMBO(gtkcombo), 1);
	/* Set maximum length of text that can be entered */
	gtk_entry_set_max_length(GTK_ENTRY(gtkentry), PATHSIZE);
	/* Return the selected item in list */
	*textinentry = gtk_entry_get_text(GTK_ENTRY(gtkentry));

	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialogfileselection)->vbox), GTK_WIDGET(gnomefileentry), TRUE, TRUE, 0);
	gtk_widget_show(gnomefileentry);

	gtk_widget_grab_focus(gtkentry);

  gtk_signal_connect(GTK_OBJECT(dialogfileselection),
                     "clicked",
                     GTK_SIGNAL_FUNC(dialog_clicked_func_cb),
                     data);

	/* Filepath should be copied to buffer whenever gtkentry loses focus */
  gtk_signal_connect(GTK_OBJECT(gtkentry),
                     "focus_out_event",
                     GTK_SIGNAL_FUNC(entry_focus_out_cb),
                     buffer);

	/* And gtkentry should get focus whenever dialog gets focus */
  gtk_signal_connect(GTK_OBJECT(dialogfileselection),
                     "focus_in_event",
                     GTK_SIGNAL_FUNC(fileentry_focus_in_cb),
                     gtkentry);

  return dialogfileselection;
}

/* gviewzip-menus ***/
